@@ -30,8 +30,12 @@ zone other.
 update add text1.other. 600 IN TXT "addition 1"
 send
 END
-[ -s ns2/nil.db.jnl ] || ret=1
-[ -s ns2/other.db.jnl ] || ret=1
+[ -s ns2/nil.db.jnl ] || {
+	echo "I: 'test -s ns2/nil.db.jnl' failed when it shouldn't have"; ret=1;
+}
+[ -s ns2/other.db.jnl ] || {
+	echo "I: 'test -s ns2/other.db.jnl' failed when it shouldn't have"; ret=1;
+}
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -41,6 +45,11 @@ $RNDCCMD freeze | sed 's/^/I:ns2 /'
 n=`expr $n + 1`
 echo "I:checking zone was dumped ($n)"
 ret=0
+for i in 1 2 3 4 5 6 7 8 9 10
+do
+	grep "addition 1" ns2/nil.db > /dev/null && break
+	sleep 1
+done
 grep "addition 1" ns2/nil.db > /dev/null 2>&1 || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
@@ -48,7 +57,9 @@ status=`expr $status + $ret`
 n=`expr $n + 1`
 echo "I:checking journal file is still present ($n)"
 ret=0
-[ -s ns2/nil.db.jnl ] || ret=1
+[ -s ns2/nil.db.jnl ] || {
+	echo "I: 'test -s ns2/nil.db.jnl' failed when it shouldn't have"; ret=1;
+}
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -62,7 +73,8 @@ update add text2.nil. 600 IN TXT "addition 2"
 send
 END
 
-$DIGCMD text2.nil. TXT | grep 'addition 2' >/dev/null && ret=1
+$DIGCMD text2.nil. TXT > dig.out.1.test$n
+grep 'addition 2' dig.out.1.test$n >/dev/null && ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -72,13 +84,14 @@ $RNDCCMD thaw | sed 's/^/I:ns2 /'
 n=`expr $n + 1`
 echo "I:checking zone now writable ($n)"
 ret=0
-$NSUPDATE -p 5300 -k ns2/session.key > /dev/null 2>&1 <<END || ret=1
+$NSUPDATE -p 5300 -k ns2/session.key > nsupdate.out.1.test$n 2>&1 <<END || ret=1
 server 10.53.0.2
 zone nil.
 update add text3.nil. 600 IN TXT "addition 3"
 send
 END
-$DIGCMD text3.nil. TXT | grep 'addition 3' >/dev/null || ret=1
+$DIGCMD text3.nil. TXT > dig.out.1.test$n
+grep 'addition 3' dig.out.1.test$n >/dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -89,6 +102,11 @@ $RNDCCMD sync nil | sed 's/^/I:ns2 /'
 n=`expr $n + 1`
 echo "I:checking zone was dumped ($n)"
 ret=0
+for i in 1 2 3 4 5 6 7 8 9 10
+do
+	grep "addition 3" ns2/nil.db > /dev/null && break
+	sleep 1
+done
 grep "addition 3" ns2/nil.db > /dev/null 2>&1 || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
@@ -96,21 +114,24 @@ status=`expr $status + $ret`
 n=`expr $n + 1`
 echo "I:checking journal file is still present ($n)"
 ret=0
-[ -s ns2/nil.db.jnl ] || ret=1
+[ -s ns2/nil.db.jnl ] || {
+	echo "I: 'test -s ns2/nil.db.jnl' failed when it shouldn't have"; ret=1;
+}
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 n=`expr $n + 1`
 echo "I:checking zone is still writable ($n)"
 ret=0
-$NSUPDATE -p 5300 -k ns2/session.key > /dev/null 2>&1 <<END || ret=1
+$NSUPDATE -p 5300 -k ns2/session.key > nsupdate.out.1.test$n 2>&1 <<END || ret=1
 server 10.53.0.2
 zone nil.
 update add text4.nil. 600 IN TXT "addition 4"
 send
 END
 
-$DIGCMD text4.nil. TXT | grep 'addition 4' >/dev/null || ret=1
+$DIGCMD text4.nil. TXT > dig.out.1.test$n
+grep 'addition 4' dig.out.1.test$n > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -121,6 +142,11 @@ $RNDCCMD sync -clean nil | sed 's/^/I:ns2 /'
 n=`expr $n + 1`
 echo "I:checking zone was dumped ($n)"
 ret=0
+for i in 1 2 3 4 5 6 7 8 9 10
+do
+	grep "addition 4" ns2/nil.db > /dev/null && break
+	sleep 1
+done
 grep "addition 4" ns2/nil.db > /dev/null 2>&1 || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
@@ -128,7 +154,9 @@ status=`expr $status + $ret`
 n=`expr $n + 1`
 echo "I:checking journal file is deleted ($n)"
 ret=0
-[ -s ns2/nil.db.jnl ] && ret=1
+[ -s ns2/nil.db.jnl ] && {
+	echo "I: 'test -s ns2/nil.db.jnl' failed when it shouldn't have"; ret=1;
+}
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -142,14 +170,17 @@ update add text5.nil. 600 IN TXT "addition 5"
 send
 END
 
-$DIGCMD text4.nil. TXT | grep 'addition 4' >/dev/null || ret=1
+$DIGCMD text4.nil. TXT > dig.out.1.test$n
+grep 'addition 4' dig.out.1.test$n >/dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 n=`expr $n + 1`
 echo "I:checking other journal files not removed ($n)"
 ret=0
-[ -s ns2/other.db.jnl ] || ret=1
+[ -s ns2/other.db.jnl ] || {
+	echo "I: 'test -s ns2/other.db.jnl' failed when it shouldn't have"; ret=1;
+}
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -159,15 +190,20 @@ $RNDCCMD sync -clean | sed 's/^/I:ns2 /'
 n=`expr $n + 1`
 echo "I:checking all journals removed ($n)"
 ret=0
-[ -s ns2/nil.db.jnl ] && ret=1
-[ -s ns2/other.db.jnl ] && ret=1
+[ -s ns2/nil.db.jnl ] && {
+	echo "I: 'test -s ns2/nil.db.jnl' succeeded when it shouldn't have"; ret=1;
+}
+[ -s ns2/other.db.jnl ] && {
+	echo "I: 'test -s ns2/other.db.jnl' succeeded when it shouldn't have"; ret=1;
+}
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 n=`expr $n + 1`
 echo "I:checking that freezing static zones is not allowed ($n)"
 ret=0
-$RNDCCMD freeze static 2>&1 | grep 'not dynamic' > /dev/null || ret=1
+$RNDCCMD freeze static > rndc.out.1.test$n 2>&1
+grep 'not dynamic' rndc.out.1.test$n > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -175,14 +211,21 @@ n=`expr $n + 1`
 echo "I:checking that journal is removed when serial is changed before thaw ($n)"
 ret=0
 sleep 1
-$NSUPDATE -p 5300 -k ns2/session.key > /dev/null 2>&1 <<END || ret=1
+$NSUPDATE -p 5300 -k ns2/session.key > nsupdate.out.1.test$n 2>&1 <<END || ret=1
 server 10.53.0.2
 zone other.
 update add text6.other. 600 IN TXT "addition 6"
 send
 END
-[ -s ns2/other.db.jnl ] || ret=1
+[ -s ns2/other.db.jnl ] || {
+	echo "I: 'test -s ns2/other.db.jnl' failed when it shouldn't have"; ret=1;
+}
 $RNDCCMD freeze other 2>&1 | sed 's/^/I:ns2 /'
+for i in 1 2 3 4 5 6 7 8 9 10
+do
+	grep "addition 6" ns2/other.db > /dev/null && break
+	sleep 1
+done
 serial=`awk '$3 == "serial" {print $1}' ns2/other.db`
 newserial=`expr $serial + 1`
 sed s/$serial/$newserial/ ns2/other.db > ns2/other.db.new
@@ -190,30 +233,42 @@ echo 'frozen TXT "frozen addition"' >> ns2/other.db.new
 mv -f ns2/other.db.new ns2/other.db
 $RNDCCMD thaw 2>&1 | sed 's/^/I:ns2 /'
 sleep 1
-[ -f ns2/other.db.jnl ] && ret=1
-$NSUPDATE -p 5300 -k ns2/session.key > /dev/null 2>&1 <<END || ret=1
+[ -f ns2/other.db.jnl ] && {
+	echo "I: 'test -f ns2/other.db.jnl' succeeded when it shouldn't have"; ret=1;
+}
+$NSUPDATE -p 5300 -k ns2/session.key > nsupdate.out.2.test$n 2>&1 <<END || ret=1
 server 10.53.0.2
 zone other.
 update add text7.other. 600 IN TXT "addition 7"
 send
 END
-$DIGCMD text6.other. TXT | grep 'addition 6' >/dev/null || ret=1
-$DIGCMD text7.other. TXT | grep 'addition 7' >/dev/null || ret=1
-$DIGCMD frozen.other. TXT | grep 'frozen addition' >/dev/null || ret=1
+$DIGCMD text6.other. TXT > dig.out.1.test$n
+grep 'addition 6' dig.out.1.test$n >/dev/null || ret=1
+$DIGCMD text7.other. TXT > dig.out.2.test$n
+grep 'addition 7' dig.out.2.test$n >/dev/null || ret=1
+$DIGCMD frozen.other. TXT > dig.out.3.test$n
+grep 'frozen addition' dig.out.3.test$n >/dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 n=`expr $n + 1`
 echo "I:checking that journal is kept when ixfr-from-differences is in use ($n)"
 ret=0
-$NSUPDATE -p 5300 -k ns2/session.key > /dev/null 2>&1 <<END || ret=1
+$NSUPDATE -p 5300 -k ns2/session.key > nsupdate.out.1.test$n 2>&1 <<END || ret=1
 server 10.53.0.2
 zone nil.
 update add text6.nil. 600 IN TXT "addition 6"
 send
 END
-[ -s ns2/nil.db.jnl ] || ret=1
+[ -s ns2/nil.db.jnl ] || {
+	echo "I: 'test -s ns2/nil.db.jnl' failed when it shouldn't have"; ret=1;
+}
 $RNDCCMD freeze nil 2>&1 | sed 's/^/I:ns2 /'
+for i in 1 2 3 4 5 6 7 8 9 10
+do
+	grep "addition 6" ns2/nil.db > /dev/null && break
+	sleep 1
+done
 serial=`awk '$3 == "serial" {print $1}' ns2/nil.db`
 newserial=`expr $serial + 1`
 sed s/$serial/$newserial/ ns2/nil.db > ns2/nil.db.new
@@ -221,16 +276,21 @@ echo 'frozen TXT "frozen addition"' >> ns2/nil.db.new
 mv -f ns2/nil.db.new ns2/nil.db
 $RNDCCMD thaw 2>&1 | sed 's/^/I:ns2 /'
 sleep 1
-[ -s ns2/nil.db.jnl ] || ret=1
-$NSUPDATE -p 5300 -k ns2/session.key > /dev/null 2>&1 <<END || ret=1
+[ -s ns2/nil.db.jnl ] || {
+	echo "I: 'test -s ns2/nil.db.jnl' failed when it shouldn't have"; ret=1;
+}
+$NSUPDATE -p 5300 -k ns2/session.key > nsupdate.out.2.test$n 2>&1 <<END || ret=1
 server 10.53.0.2
 zone nil.
 update add text7.nil. 600 IN TXT "addition 7"
 send
 END
-$DIGCMD text6.nil. TXT | grep 'addition 6' >/dev/null || ret=1
-$DIGCMD text7.nil. TXT | grep 'addition 7' >/dev/null || ret=1
-$DIGCMD frozen.nil. TXT | grep 'frozen addition' >/dev/null || ret=1
+$DIGCMD text6.nil. TXT > dig.out.1.test$n
+grep 'addition 6' dig.out.1.test$n > /dev/null || ret=1
+$DIGCMD text7.nil. TXT > dig.out.2.test$n
+grep 'addition 7' dig.out.2.test$n > /dev/null || ret=1
+$DIGCMD frozen.nil. TXT > dig.out.3.test$n
+grep 'frozen addition' dig.out.3.test$n >/dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -363,8 +423,8 @@ status=`expr $status + $ret`
 n=`expr $n + 1`
 echo "I:testing automatic zones are reported ($n)"
 ret=0
-$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf status > rndc.output.test$n || ret=1
-grep "number of zones: 198 (196 automatic)" rndc.output.test$n > /dev/null || ret=1
+$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf status > rndc.out.1.test$n || ret=1
+grep "number of zones: 198 (196 automatic)" rndc.out.1.test$n > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -388,26 +448,31 @@ echo "I:testing rndc with querylog command ($n)"
 ret=0
 # first enable it with querylog on option
 $RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf querylog on >/dev/null 2>&1 || ret=1
+grep "query logging is now on" ns4/named.run > /dev/null || ret=1
 # query for builtin and check if query was logged
 $DIG @10.53.0.4 -p 5300 -c ch -t txt foo12345.bind > /dev/null || ret 1
-grep "query logging is now on" ns4/named.run > /dev/null || ret=1
 grep "query: foo12345.bind CH TXT" ns4/named.run > /dev/null || ret=1
 # toggle query logging and check again
 $RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf querylog > /dev/null 2>&1 || ret=1
+grep "query logging is now off" ns4/named.run > /dev/null || ret=1
 # query for another builtin zone and check if query was logged
 $DIG @10.53.0.4 -p 5300 -c ch -t txt foo9876.bind > /dev/null || ret 1
-grep "query logging is now off" ns4/named.run > /dev/null || ret=1
 grep "query: foo9876.bind CH TXT" ns4/named.run > /dev/null && ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
+RNDCCMD4="$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf"
 n=`expr $n + 1`
 echo "I:testing rndc nta time limits ($n)"
 ret=0
-$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf nta -l 2h nta1.example 2>&1 | grep "Negative trust anchor added" > /dev/null || ret=1
-$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf nta -l 1d nta2.example 2>&1 | grep "Negative trust anchor added" > /dev/null || ret=1
-$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf nta -l 1w nta3.example 2>&1 | grep "Negative trust anchor added" > /dev/null || ret=1
-$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf nta -l 8d nta4.example 2>&1 | grep "NTA lifetime cannot exceed one week" > /dev/null || ret=1
+$RNDCCMD4 nta -l 2h nta1.example > rndc.out.1.test$n 2>&1
+grep "Negative trust anchor added" rndc.out.1.test$n > /dev/null || ret=1
+$RNDCCMD4 nta -l 1d nta2.example > rndc.out.2.test$n 2>&1
+grep "Negative trust anchor added" rndc.out.2.test$n > /dev/null || ret=1
+$RNDCCMD4 nta -l 1w nta3.example > rndc.out.3.test$n 2>&1
+grep "Negative trust anchor added" rndc.out.3.test$n > /dev/null || ret=1
+$RNDCCMD4 nta -l 8d nta4.example > rndc.out.4.test$n 2>&1
+grep "NTA lifetime cannot exceed one week" rndc.out.4.test$n > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -416,8 +481,8 @@ do
 	n=`expr $n + 1`
 	echo "I:testing rndc buffer size limits (size=${i}) ($n)"
 	ret=0
-	$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf testgen ${i} 2>&1 > rndc.output.test$n || ret=1
-	actual_size=`$GENCHECK rndc.output.test$n`
+	$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf testgen ${i} 2>&1 > rndc.out.$i.test$n || ret=1
+	actual_size=`$GENCHECK rndc.out.$i.test$n`
 	if [ "$?" = "0" ]; then
 	    expected_size=`expr $i + 1`
 	    if [ $actual_size != $expected_size ]; then ret=1; fi
@@ -432,16 +497,16 @@ done
 n=`expr $n + 1`
 echo "I:testing rndc -r (show result) ($n)"
 ret=0
-$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf -r testgen 0 2>&1 > rndc.output.test$n || ret=1
-grep "ISC_R_SUCCESS 0" rndc.output.test$n > /dev/null || ret=1
+$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf -r testgen 0 2>&1 > rndc.out.1.test$n || ret=1
+grep "ISC_R_SUCCESS 0" rndc.out.1.test$n > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 n=`expr $n + 1`
 echo "I:testing rndc with a token containing a space ($n)"
 ret=0
-$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf -r flush '"view with a space"' 2>&1 > rndc.output.test$n || ret=1
-grep "not found" rndc.output.test$n > /dev/null && ret=1
+$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf -r flush '"view with a space"' 2>&1 > rndc.out.1.test$n || ret=1
+grep "not found" rndc.out.1.test$n > /dev/null && ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -452,8 +517,8 @@ $RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf reconfig > /dev/null || ret=1
 sleep 1
 mv ns4/named.conf ns4/named.conf.save
 echo "error error error" >> ns4/named.conf
-$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf reconfig > rndc.output.test$n 2>&1 && ret=1
-grep "rndc: 'reconfig' failed: unexpected token" rndc.output.test$n > /dev/null || ret=1
+$RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf reconfig > rndc.out.1.test$n 2>&1 && ret=1
+grep "rndc: 'reconfig' failed: unexpected token" rndc.out.1.test$n > /dev/null || ret=1
 mv ns4/named.conf.save ns4/named.conf
 sleep 1
 $RNDC -s 10.53.0.4 -p 9956 -c ns4/key6.conf reconfig > /dev/null || ret=1
@@ -473,8 +538,8 @@ status=`expr $status + $ret`
 n=`expr $n + 1`
 echo "I:test rndc status shows running on ($n)"
 ret=0
-$RNDC -s 10.53.0.5 -p 9953 -c ../common/rndc.conf status > rndc.output.test$n 2>&1 || ret=1
-grep "^running on " rndc.output.test$n > /dev/null || ret=1
+$RNDC -s 10.53.0.5 -p 9953 -c ../common/rndc.conf status > rndc.out.1.test$n 2>&1 || ret=1
+grep "^running on " rndc.out.1.test$n > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
@@ -485,7 +550,7 @@ cur=`awk 'BEGIN {l=0} /^/ {l++} END { print l }' ns6/named.run`
 cp ns6/named.conf ns6/named.conf.save
 echo "zone \"huge.zone\" { type master; file \"huge.zone.db\"; };" >> ns6/named.conf
 echo " I:reloading config"
-$RNDC -s 10.53.0.6 -p 9953 -c ../common/rndc.conf reconfig > rndc.output.test$n 2>&1 || ret=1
+$RNDC -s 10.53.0.6 -p 9953 -c ../common/rndc.conf reconfig > rndc.out.1.test$n 2>&1 || ret=1
 if [ $ret != 0 ]; then echo " I:failed"; fi
 status=`expr $status + $ret`
 sleep 1
@@ -497,8 +562,8 @@ status=`expr $status + $ret`
 
 n=`expr $n + 1`
 echo " I:check if query for the zone returns SERVFAIL ($n)"
-$DIG @10.53.0.6 -p 5300 -t soa huge.zone > dig.out.test$n
-grep "SERVFAIL" dig.out.test$n > /dev/null || ret=1
+$DIG @10.53.0.6 -p 5300 -t soa huge.zone > dig.out.1.test$n
+grep "SERVFAIL" dig.out.1.test$n > /dev/null || ret=1
 if [ $ret != 0 ]; then echo " I:failed (ignored)"; ret=0; fi
 status=`expr $status + $ret`
 
@@ -520,8 +585,8 @@ status=`expr $status + $ret`
 
 n=`expr $n + 1`
 echo " I:check if query for the zone returns NOERROR ($n)"
-$DIG @10.53.0.6 -p 5300 -t soa huge.zone > dig.out.test$n
-grep "NOERROR" dig.out.test$n > /dev/null || ret=1
+$DIG @10.53.0.6 -p 5300 -t soa huge.zone > dig.out.1.test$n
+grep "NOERROR" dig.out.1.test$n > /dev/null || ret=1
 if [ $ret != 0 ]; then echo " I:failed"; fi
 status=`expr $status + $ret`
 
@@ -544,7 +609,7 @@ if [ -x "$PYTHON" ]; then
     n=`expr $n + 1`
     echo "I:test rndc python bindings ($n)"
     ret=0
-    $PYTHON > rndc.output.test$n << EOF
+    $PYTHON > python.out.1.test$n << EOF
 import sys
 sys.path.insert(0, '../../../../bin/python')
 from isc import *
@@ -552,7 +617,7 @@ r = rndc(('10.53.0.5', 9953), 'hmac-sha256', '1234abcd8765')
 result = r.call('status')
 print(result['text'])
 EOF
-    grep 'server is up and running' rndc.output.test$n > /dev/null 2>&1 || ret=1
+    grep 'server is up and running' python.out.1.test$n > /dev/null 2>&1 || ret=1
     if [ $ret != 0 ]; then echo "I:failed"; fi
     status=`expr $status + $ret`
 fi
@@ -560,8 +625,8 @@ fi
 n=`expr $n + 1`
 echo "I:check 'rndc \"\"' is handled ($n)"
 ret=0
-$RNDCCMD "" > rndc.out.test$n 2>&1 && ret=1
-grep "rndc: '' failed: failure" rndc.out.test$n > /dev/null
+$RNDCCMD "" > rndc.out.1.test$n 2>&1 && ret=1
+grep "rndc: '' failed: failure" rndc.out.1.test$n > /dev/null
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
