@@ -1,21 +1,11 @@
 /*
- * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2003-2007, 2009, 2013-2016  Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-/* $Id: hash.c,v 1.16 2009/09/01 00:22:28 jinmei Exp $ */
-
 /*! \file
  * Some portion of this code was derived from universal hash function
  * libraries of Rice University.
@@ -100,7 +90,8 @@ struct isc_hash {
 
 static isc_mutex_t createlock;
 static isc_once_t once = ISC_ONCE_INIT;
-static isc_hash_t *hash = NULL;
+
+LIBISC_EXTERNAL_DATA isc_hash_t *isc_hashctx = NULL;
 
 static unsigned char maptolower[] = {
 	0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
@@ -193,12 +184,8 @@ isc_hash_ctxcreate(isc_mem_t *mctx, isc_entropy_t *entropy,
 	hctx->vectorlen = vlen;
 	hctx->rndvector = rv;
 
-#ifdef BIND9
 	if (entropy != NULL)
 		isc_entropy_attach(entropy, &hctx->entropy);
-#else
-	UNUSED(entropy);
-#endif
 
 	*hctxp = hctx;
 	return (ISC_R_SUCCESS);
@@ -223,14 +210,15 @@ isc_hash_create(isc_mem_t *mctx, isc_entropy_t *entropy, size_t limit) {
 	isc_result_t result = ISC_R_SUCCESS;
 
 	REQUIRE(mctx != NULL);
-	INSIST(hash == NULL);
+	INSIST(isc_hashctx == NULL);
 
 	RUNTIME_CHECK(isc_once_do(&once, initialize_lock) == ISC_R_SUCCESS);
 
 	LOCK(&createlock);
 
-	if (hash == NULL)
-		result = isc_hash_ctxcreate(mctx, entropy, limit, &hash);
+	if (isc_hashctx == NULL)
+		result = isc_hash_ctxcreate(mctx, entropy, limit,
+					    &isc_hashctx);
 
 	UNLOCK(&createlock);
 
@@ -244,8 +232,7 @@ isc_hash_ctxinit(isc_hash_t *hctx) {
 	if (hctx->initialized == ISC_TRUE)
 		goto out;
 
-	if (hctx->entropy) {
-#ifdef BIND9
+	if (hctx->entropy != NULL) {
 		isc_result_t result;
 
 		result = isc_entropy_getdata(hctx->entropy,
@@ -253,9 +240,6 @@ isc_hash_ctxinit(isc_hash_t *hctx) {
 					     (unsigned int)hctx->vectorlen,
 					     NULL, 0);
 		INSIST(result == ISC_R_SUCCESS);
-#else
-		INSIST(0);
-#endif
 	} else {
 		isc_uint32_t pr;
 		size_t i, copylen;
@@ -283,9 +267,9 @@ isc_hash_ctxinit(isc_hash_t *hctx) {
 
 void
 isc_hash_init(void) {
-	INSIST(hash != NULL && VALID_HASH(hash));
+	INSIST(isc_hashctx != NULL && VALID_HASH(isc_hashctx));
 
-	isc_hash_ctxinit(hash);
+	isc_hash_ctxinit(isc_hashctx);
 }
 
 void
@@ -311,10 +295,8 @@ destroy(isc_hash_t **hctxp) {
 	isc_refcount_destroy(&hctx->refcnt);
 
 	mctx = hctx->mctx;
-#ifdef BIND9
 	if (hctx->entropy != NULL)
 		isc_entropy_detach(&hctx->entropy);
-#endif
 	if (hctx->rndvector != NULL)
 		isc_mem_put(mctx, hctx->rndvector, hctx->vectorlen);
 
@@ -346,12 +328,12 @@ void
 isc_hash_destroy(void) {
 	unsigned int refs;
 
-	INSIST(hash != NULL && VALID_HASH(hash));
+	INSIST(isc_hashctx != NULL && VALID_HASH(isc_hashctx));
 
-	isc_refcount_decrement(&hash->refcnt, &refs);
+	isc_refcount_decrement(&isc_hashctx->refcnt, &refs);
 	INSIST(refs == 0);
 
-	destroy(&hash);
+	destroy(&isc_hashctx);
 }
 
 static inline unsigned int
@@ -393,10 +375,24 @@ unsigned int
 isc_hash_calc(const unsigned char *key, unsigned int keylen,
 	      isc_boolean_t case_sensitive)
 {
-	INSIST(hash != NULL && VALID_HASH(hash));
-	REQUIRE(keylen <= hash->limit);
+	INSIST(isc_hashctx != NULL && VALID_HASH(isc_hashctx));
+	REQUIRE(keylen <= isc_hashctx->limit);
 
-	return (hash_calc(hash, key, keylen, case_sensitive));
+	return (hash_calc(isc_hashctx, key, keylen, case_sensitive));
+}
+
+void
+isc__hash_setvec(const isc_uint16_t *vec) {
+	int i;
+	hash_random_t *p;
+
+	if (isc_hashctx == NULL)
+		return;
+
+	p = isc_hashctx->rndvector;
+	for (i = 0; i < 256; i++) {
+		p[i] = vec[i];
+	}
 }
 
 static isc_uint32_t fnv_offset_basis;
@@ -414,6 +410,26 @@ fnv_initialize(void) {
 	}
 }
 
+const void *
+isc_hash_get_initializer(void) {
+	RUNTIME_CHECK(isc_once_do(&fnv_once, fnv_initialize) == ISC_R_SUCCESS);
+
+	return (&fnv_offset_basis);
+}
+
+void
+isc_hash_set_initializer(const void *initializer) {
+	REQUIRE(initializer != NULL);
+
+	/*
+	 * Ensure that fnv_initialize() is not called after
+	 * isc_hash_set_initializer() is called.
+	 */
+	RUNTIME_CHECK(isc_once_do(&fnv_once, fnv_initialize) == ISC_R_SUCCESS);
+
+	fnv_offset_basis = *((const unsigned int *) initializer);
+}
+
 isc_uint32_t
 isc_hash_function(const void *data, size_t length,
 		  isc_boolean_t case_sensitive,
@@ -438,7 +454,7 @@ isc_hash_function(const void *data, size_t length,
 	/*
 	 * Fowler-Noll-Vo FNV-1a hash function.
 	 *
-	 * NOTE: A random fnv_offset_basis is used by default to avoid
+	 * NOTE: A random FNV offset basis is used by default to avoid
 	 * collision attacks as the hash function is reversible. This
 	 * makes the mapping non-deterministic, but the distribution in
 	 * the domain is still uniform.
@@ -505,7 +521,7 @@ isc_hash_function_reverse(const void *data, size_t length,
 	/*
 	 * Fowler-Noll-Vo FNV-1a hash function.
 	 *
-	 * NOTE: A random fnv_offset_basis is used by default to avoid
+	 * NOTE: A random FNV offset basis is used by default to avoid
 	 * collision attacks as the hash function is reversible. This
 	 * makes the mapping non-deterministic, but the distribution in
 	 * the domain is still uniform.
